interpArray <- function(point, dataArray, timeRes = 1)
# returns bilinear or trilinear interpolated grid value at point
{
  # first index of dataArray is time-dimension, second age-dimension, third duration
  # first entry (1, 1, 1) is assumed to be origin (0, 0, 0)
  # timeRes applies to all dimensions
  
  intT   <- trunc(point/timeRes)       # integer fraction
  remT   <- point - intT * timeRes     # remainder
  idx    <- intT + 1
  
  if(is.na(dim(dataArray)[3])){
    #dataArray is 2D -> interpolate bilinear
    ret  <- dataArray[idx[1], idx[2]] +
               remT[1] * (dataArray[idx[1] + 1, idx[2]    ] - dataArray[idx[1], idx[2]]) +
               remT[2] * (dataArray[idx[1]    , idx[2] + 1] - dataArray[idx[1], idx[2]]) 
     
  }
  else{
    #dataArray is 3D  -> interpolate trilinear
    ret  <- dataArray[idx[1], idx[2], idx[3]] +
               remT[1] * (dataArray[idx[1] + 1, idx[2]    , idx[3]    ] - dataArray[idx[1], idx[2], idx[3]]) +
               remT[2] * (dataArray[idx[1]    , idx[2] + 1, idx[3]    ] - dataArray[idx[1], idx[2], idx[3]]) +
               remT[3] * (dataArray[idx[1]    , idx[2]    , idx[3] + 1] - dataArray[idx[1], idx[2], idx[3]])
  }
  return(ret)
}
