sampleLine <- function(start, end, timeRes = 1)
# Siddon raytracer
{
   tin   <- start[1]
   tout  <- end  [1]
   ain   <- start[2]
   aout  <- end  [2]
   din   <- start[3]
   dout  <- end  [3]

   #(tn, am, dk) are indices of first lexis voxel that gets weight
   tn    <- trunc(tin  / timeRes)
   am    <- trunc(ain  / timeRes)
   dk    <- trunc(din  / timeRes)

   rem_t <- tin - tn*timeRes
   rem_a <- ain - am*timeRes
   rem_d <- din - dk*timeRes

   #(tN, aM, dK) are indices of last lexis voxel that gets weight
   tN    <- trunc(tout   / timeRes)
   aM    <- trunc(aout   / timeRes)
   dK    <- trunc(dout   / timeRes)

   #decide wether one or more voxels get weight
   if(tn == tN & am == aM & dk == dK)
   {
      #only one voxel gets weight
      nIntersectedVoxels <- 1
      pnts               <- matrix(0, 3, 2, dimnames = list(c("t", "a", "d"), 1:2))
      pnts[ ,1]          <- start
      pnts[ ,2]          <- end
      wghts              <- tout - tin
   }
   else
   {
      #parameterize the intersection points with the voxel planes
      nIntersect <- c(tN - tn, aM - am, dK - dk)
      if(nIntersect[1] > 0)
         alphaT   <- ((1:nIntersect[1])*timeRes - rem_t) / (tout - tin)
      else
         alphaT   <- NA
      if(nIntersect[2] > 0)
         alphaA   <- ((1:nIntersect[2])*timeRes - rem_a) / (tout - tin)
      else
         alphaA   <- NA
      if(nIntersect[3] > 0)
         alphaD   <- ((1:nIntersect[3])*timeRes - rem_d) / (tout - tin)
      else
         alphaD   <- NA

      alpha       <- sort(unique(na.omit(c(0, alphaT, alphaA, alphaD))))
      #sometimes alpha contains 1.0 already, if not add
      if(!isTRUE(all.equal(1.0, alpha[length(alpha)])))
         alpha <- c(alpha, 1.0)
      #now the vector alpha contains all elements of the set A^star
      
      nIntersectedVoxels  <- length(alpha) - 1
      nPoints             <- length(alpha)
      pnts                <- matrix(0, 3, nPoints, dimnames = list(c("t", "a", "d"), 1:nPoints))
      wghts               <- numeric(nIntersectedVoxels)

      direction <- c(1, 1, ifelse(dout > 0, 1, 0))
      for(intersectNr in 1:nPoints){
         pnts[ , intersectNr]  <- c(tin, ain, din) + alpha[intersectNr] * (tout - tin) * direction
      }
      wghts <- (c(alpha, 0) - c(0, alpha))[2:nPoints] * (tout - tin)
   }#end if-else one or more voxels are intersected
   list( nIntersectedVoxels = nIntersectedVoxels,
                     points = pnts, 
                    weights = wghts)
}