maxAge <- 120     # age when all persons are dead
nPers  <- 200000

pmat   <- matrix(data = 0, nrow = nPers, ncol = 4) # matrix of results
F1     <- numeric(maxAge+1)

#incidence i
fct_i <- function(a){
   if(a < 20) return(0)
   else return(0.005/(1-0.005*(a-20)))
}

#integral of i
fct_I  <- function(a){
   if(a < 20) return(0)
   else return(-log(1-0.005*(a-20)))
}

#integral of mortality m0
fct_M0 <- function(a){
   10*exp(-10)*(exp(0.1 * a) - 1)
}

#calculate the cdf of the first failure time
for(t in 0:maxAge){
  F1[t+1]  <- 1-exp(-fct_I(t) - fct_M0(t))
}

set.seed(1234)
# Algorithm 1
for(pNr in 1:nPers){
   rr <- runif(1)
   aprime <- which.min(rr > F1) - 2
   changeAge <- aprime + (rr - F1[aprime+1]) / (F1[aprime+2] - F1[aprime+1])
   thisT <- changeAge
   thisInc <- fct_i(changeAge)
   if(runif(1) < thisInc / (thisInc + exp(-10 + 0.1*changeAge))){
      #is diagnosis
      adi <- changeAge
      ade <- NA
   }
   else
   {
      #is death
      adi <- NA
      ade <- changeAge
   }
   pmat[pNr, ] <- c(pNr, runif(1), adi, ade)
}

#for those who have not died (ade == NA), 
#calculate the second failure time
for(pid in 1:nPers){
   if(is.na(pmat[pid, 4])){
     adi    <- pmat[pid, 3]
     maxT   <- maxAge - ceiling(adi)

     F3     <- numeric(maxT+1)
     for(t in 0:(maxT)){
        F3[t+1]  <- 1-exp(-(fct_M0(t + adi) - fct_M0(adi))) 
     }
     rr <- runif(1)
     aprime <- which.min(rr > F3) - 2
     ade <- aprime + (rr - F3[aprime+1]) / (F3[aprime+2] - F3[aprime+1])
     pmat[pid, 4] <- adi + ade
   }
}

#calculate diseased and persons alive from the pmat
empPrev <- function(a){
   c <- sum(pmat[ ,3] <= a & pmat[ ,4] > a, na.rm = TRUE)
   n <- sum(pmat[ ,4] > a)
   p <- c/n
   s <- sqrt(p*(1-p)/n)
   list(p = p, s = s)
}

#prevalence is p(a) = 0.005*max(0, a-20)
matplot(0:83, 0.005*c(rep(0, 21), (21:83)-20), type="l", 
          xlab = "Age (years)", ylab = "Prevalence", col = "blue")
x    <- 5*1:16
meas <- sapply(x, FUN = empPrev)
p <- as.numeric(meas[1, ]); s <- as.numeric(meas[2, ])
matplot(x, p, type = "p", pch=3, add=TRUE)
legend("topleft", legend=c("Simulation", "Analytic"), pch=c(3,-1), 
  lty = c(0, 1), col=c("black", "blue"))
