source("interpArray.R")
source("sampleLine.R")
source("lineIntegral.R")
source("inc.sim2.R")
source("m0.sim2.R")

dt      <- 2
maxAge  <- 120
ages    <- seq(0, maxAge, by = dt)
maxTime <- 180
times   <- seq(0, maxTime, by = dt)

i0      <- sapply(ages, FUN = fct_inc)           # i(0, a)
incHaz  <- rep(1, length(times))
incMat  <- outer(incHaz, i0)

m0      <- sapply(ages, FUN = fct_m0, t = 0)
m0Haz   <- exp(times*log(0.998))
m0Mat   <- outer(m0Haz, m0)

riskMat <- m0Mat + incMat

nPersInYear <- rep(2000, 60)
pmat        <- matrix(data = 0, nrow = sum(nPersInYear), ncol = 4)


set.seed(1234)
pid <- 1
for(dob in 1:60)
{
   start      <- c(dob, 0, 0)
   end        <- start + (maxAge-1)*c(1, 1, 0)
   lineIntgrl <- lineIntegral(start, end, riskMat, timeRes = dt)
   F1         <- 1 - exp(-lineIntgrl$values)

   for(pNr in 1:nPersInYear[dob]){
      u <- runif(1)
      p <- which.min(u > F1)
      # backward interpolation
      changeAge <- (lineIntgrl$points)[2, p-1] + (u - F1[p-1])*
                        ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F1[p] - F1[p-1])
      thisT <- dob + changeAge
      thisInc <- fct_inc(changeAge)
      if(runif(1) < thisInc/(thisInc + fct_m0(thisT, changeAge))){
         #is diagnosis
         adi <- changeAge
         ade <- NA
      }
      else
      {
         #is death
         adi <- NA
         ade <- changeAge
      }
      pmat[pid, ] <- c(pid, dob+runif(1), adi, ade)
      pid <- pid + 1   
   }  
}

R      <- 0.04* (seq(0, 46, by = dt) - 5)^2 + 1
m1Arr  <- outer(m0Mat, R)
for(pid in 1:sum(nPersInYear)){
   if(is.na(pmat[pid, 4])){
     dob    <- pmat[pid, 2]
     adi    <- pmat[pid, 3]
     maxDur <- min(maxAge - ceiling(adi), 
                  45, maxTime - ceiling(dob) - ceiling(adi))
     
     start <- c(dob + adi, adi, 0)
     end   <- start + (maxDur-1) * c(1, 1, 1)
     lineIntgrl <- lineIntegral(start, end, m1Arr, timeRes = dt)
     F3 <- 1 - exp(-lineIntgrl$values)

     u <- runif(1)
     p <- which.min(u > F3)
     ade <- (lineIntgrl$points)[2, p-1] + (u - F3[p-1])*
                  ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F3[p] - F3[p-1])
     pmat[pid, 4] <- ade
   }
}

gotIll <- !is.na(pmat[ ,3])

# number of diseased persons
sum(gotIll)

# age at death
median(pmat[ gotIll, 4])
median(pmat[!gotIll, 4])

# duration of the disease
summary(pmat[gotIll, 4] - pmat[gotIll, 3])

# cross-section at t = 100
source("cutCases.R")
mm <- cutCases(pmat, 100, 5*0:24)

sum(mm)      # number of persons alive at t = 100
sum(mm[2, ]) # number of diseased persons at t = 100
