source("interpArray.R")
source("sampleLine.R")
source("inc.sim3.R")
source("m0.sim3.R")
source("lineIntegral.R")

dt      <- 1
maxAge  <- 110
ages    <- seq(0, maxAge, by = dt)
maxTime <- 120
times   <- seq(0, maxTime, by = dt)

i0      <- sapply(ages, FUN = fct_inc)       #  i(0, a)
incHaz  <- rep(1, length(times))
incMat  <- outer(incHaz, i0)

m0      <- sapply(ages, FUN = fct_m0, t = 0)  # m0(0, a)
m0Haz   <- exp(times*log(0.9964))             # geom mean of mort reduction
m0Mat   <- outer(m0Haz, m0)

riskMat <- m0Mat + incMat

nPers   <- 100000
pmat    <- matrix(data = NA, nrow = nPers, ncol = 4)

set.seed(1234)

dob        <- 5                               # t = 0 is 1940  
start      <- c(dob, 0, 0)
end        <- start + (maxAge-1)*c(1, 1, 0)
lineIntgrl <- lineIntegral(start, end, riskMat, timeRes = dt)
F1         <- 1 - exp(-lineIntgrl$values)

for(pid in 1:nPers){
   u <- runif(1)
   p <- which.min(u > F1)
   # backward interpolation
   changeAge <- (lineIntgrl$points)[2, p-1] + (u - F1[p-1])*
                        ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F1[p] - F1[p-1])
   thisT <- dob + changeAge
   thisInc <- fct_inc(changeAge)
   if(runif(1) < thisInc/(thisInc + fct_m0(thisT, changeAge))){
      #is diagnosis
      adi <- changeAge
      ade <- NA
   }
   else
   {
      #is death
      adi <- NA
      ade <- changeAge
   }
   pmat[pid, ] <- c(pid, dob+runif(1), adi, ade)
}  

a        <-     c( 20,  50, 60)
logSMR.a <- log(c(6.4, 2.6, 1.0))
HR.a     <- exp(approx(a, logSMR.a, ages, rule = 2)$y)

duratn   <-     c(0.5, 3,   7.5, 15, 20)
logSMR.d <- log(c(7.7, 3.2, 2.4, 1.8, 1))
HR.d     <- exp(approx(duratn, logSMR.d, 0:80, rule = 2)$y)

m1       <- m0 * HR.a * 1.2 * 1.6 #put in age, sex and region
m1Mat    <- outer(m0Haz, m1)      #put in calendar time
m1Arr    <- outer(m1Mat, HR.d)    #put in duration

for(pid in 1:nPers){
   if(!is.na(pmat[pid, 3])){
     # simulate ade for ill subjects
     dob    <- pmat[pid, 2]
     adi    <- pmat[pid, 3]
     maxDur <- min(maxAge - ceiling(adi), 
                  80, maxTime - ceiling(dob) - ceiling(adi))
     
     start <- c(dob + adi, adi, 0)
     end   <- start + (maxDur-1) * c(1, 1, 1)
     lineIntgrl <- lineIntegral(start, end, m1Arr, timeRes = dt)
     F3 <- 1 - exp(-lineIntgrl$values)

     u <- runif(1)
     p <- which.min(u > F3)
     ade <- (lineIntgrl$points)[2, p-1] + (u - F3[p-1])*
                  ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F3[p] - F3[p-1])
     pmat[pid, 4] <- ade
   }
}
isIll <- !is.na(pmat[ ,3])
sum(isIll)
#Age of onset
summary(pmat[isIll ,3])
#Age at death ill/healthy
summary(pmat[isIll ,4]); summary(pmat[!isIll ,4])
#Duration of SLE
summary(pmat[isIll, 4] - pmat[isIll, 3])


adi <- pmat[isIll, 3]
dur <- pmat[isIll, 4] - adi
round(prop.table(table(cut(adi, c(5, 35, 45, 60, 90),    right = FALSE), 
                 cut(dur, c(0, 1, 2.5, 5, 10, 25, 70.5), right = FALSE),
                 dnn = list("adi", "dur")), margin = 1
), 3)
