source("interpArray.R")
source("sampleLine.R")
source("lineIntegral.R")
source("inc.sim2.R")
source("m0.sim2.R")

dt      <- 2
maxAge  <- 120
ages    <- seq(0, maxAge, by = dt)
maxTime <- 180
times   <- seq(0, maxTime, by = dt)

nPersInYear <- rep(2000, 60)

# allocate matrices for the populations (basecase, intervention A and B)
pmat.BC     <- matrix(data = 0, nrow = sum(nPersInYear), ncol = 4)
pmat.A      <- matrix(data = 0, nrow = sum(nPersInYear), ncol = 4)
pmat.B      <- matrix(data = 0, nrow = sum(nPersInYear), ncol = 4)


set.seed(12345)

## Basecase
i0      <- sapply(ages, FUN = fct_inc)           # i(0, a)
incHaz  <- rep(1, length(times))
incMat  <- outer(incHaz, i0)

m0      <- sapply(ages, FUN = fct_m0, t = 0)
m0Haz   <- exp(times*log(0.998))
m0Mat   <- outer(m0Haz, m0)

riskMat <- m0Mat + incMat

pid <- 1
for(dob in 1:60)
{
   start      <- c(dob, 0, 0)
   end        <- start + (maxAge-1)*c(1, 1, 0)
   lineIntgrl <- lineIntegral(start, end, riskMat, timeRes = dt)
   F1         <- 1 - exp(-lineIntgrl$values)

   for(pNr in 1:nPersInYear[dob]){
      u <- runif(1)
      p <- which.min(u > F1)
      # backward interpolation
      changeAge <- (lineIntgrl$points)[2, p-1] + (u - F1[p-1])*
                        ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F1[p] - F1[p-1])
      thisT <- dob + changeAge
      thisInc <- fct_inc(changeAge)
      if(runif(1) < thisInc/(thisInc + fct_m0(thisT, changeAge))){
         #is diagnosis
         adi <- changeAge
         ade <- NA
      }
      else
      {
         #is death
         adi <- NA
         ade <- changeAge
      }
      pmat.BC[pid, ] <- c(pid, dob+runif(1), adi, ade)
      pid <- pid + 1   
   }  
}

R      <- 0.04* (seq(0, 46, by = dt) - 5)^2 + 1
m1Arr  <- outer(m0Mat, R)
for(pid in 1:sum(nPersInYear)){
   if(is.na(pmat.BC[pid, 4])){
     dob    <- pmat.BC[pid, 2]
     adi    <- pmat.BC[pid, 3]
     maxDur <- min(maxAge - ceiling(adi), 
                  45, maxTime - ceiling(dob) - ceiling(adi))
     
     start <- c(dob + adi, adi, 0)
     end   <- start + (maxDur-1) * c(1, 1, 1)
     lineIntgrl <- lineIntegral(start, end, m1Arr, timeRes = dt)
     F3 <- 1 - exp(-lineIntgrl$values)

     u <- runif(1)
     p <- which.min(u > F3)
     ade <- (lineIntgrl$points)[2, p-1] + (u - F3[p-1])*
                  ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F3[p] - F3[p-1])
     pmat.BC[pid, 4] <- ade
   }
}

## Intervention A
incHaz  <- incHaz*c(seq(1, 0.85, length = 5), rep(0.85, 86)) 
incMat  <- outer(incHaz, i0)

riskMat <- m0Mat + incMat

pid <- 1
for(dob in 1:60)
{
   start      <- c(dob, 0, 0)
   end        <- start + (maxAge-1)*c(1, 1, 0)
   lineIntgrl <- lineIntegral(start, end, riskMat, timeRes = dt)
   F1         <- 1 - exp(-lineIntgrl$values)

   for(pNr in 1:nPersInYear[dob]){
      u <- runif(1)
      p <- which.min(u > F1)
      # backward interpolation
      changeAge <- (lineIntgrl$points)[2, p-1] + (u - F1[p-1])*
                        ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F1[p] - F1[p-1])
      thisT <- dob + changeAge
      thisInc <- fct_inc(changeAge)
      if(runif(1) < thisInc/(thisInc + fct_m0(thisT, changeAge))){
         #is diagnosis
         adi <- changeAge
         ade <- NA
      }
      else
      {
         #is death
         adi <- NA
         ade <- changeAge
      }
      pmat.A[pid, ] <- c(pid, dob+runif(1), adi, ade)
      pid <- pid + 1   
   }  
}

for(pid in 1:sum(nPersInYear)){
   if(is.na(pmat.A[pid, 4])){
     dob    <- pmat.A[pid, 2]
     adi    <- pmat.A[pid, 3]
     maxDur <- min(maxAge - ceiling(adi), 
                  45, maxTime - ceiling(dob) - ceiling(adi))
     
     start <- c(dob + adi, adi, 0)
     end   <- start + (maxDur-1) * c(1, 1, 1)
     lineIntgrl <- lineIntegral(start, end, m1Arr, timeRes = dt)
     F3 <- 1 - exp(-lineIntgrl$values)

     u <- runif(1)
     p <- which.min(u > F3)
     ade <- (lineIntgrl$points)[2, p-1] + (u - F3[p-1])*
                  ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F3[p] - F3[p-1])
     pmat.A[pid, 4] <- ade
   }
}

## Intervention B
i0      <- sapply(ages, FUN = fct_inc)           # i(0, a)
incHaz  <- rep(1, length(times))
incMat  <- outer(incHaz, i0)

riskMat <- m0Mat + incMat

pid <- 1
for(dob in 1:60)
{
   start      <- c(dob, 0, 0)
   end        <- start + (maxAge-1)*c(1, 1, 0)
   lineIntgrl <- lineIntegral(start, end, riskMat, timeRes = dt)
   F1         <- 1 - exp(-lineIntgrl$values)

   for(pNr in 1:nPersInYear[dob]){
      u <- runif(1)
      p <- which.min(u > F1)
      # backward interpolation
      changeAge <- (lineIntgrl$points)[2, p-1] + (u - F1[p-1])*
                        ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F1[p] - F1[p-1])
      thisT <- dob + changeAge
      thisInc <- fct_inc(changeAge)
      if(runif(1) < thisInc/(thisInc + fct_m0(thisT, changeAge))){
         #is diagnosis
         adi <- changeAge
         ade <- NA
      }
      else
      {
         #is death
         adi <- NA
         ade <- changeAge
      }
      pmat.B[pid, ] <- c(pid, dob+runif(1), adi, ade)
      pid <- pid + 1   
   }  
}


R      <- R * c(rep(1, 3), 0.85, 0.72, 0.6, 0.5, rep(0.5, 17))
m1Arr  <- outer(m0Mat, R)
for(pid in 1:sum(nPersInYear)){
   if(is.na(pmat.B[pid, 4])){
     dob    <- pmat.B[pid, 2]
     adi    <- pmat.B[pid, 3]
     maxDur <- min(maxAge - ceiling(adi), 
                  45, maxTime - ceiling(dob) - ceiling(adi))
     
     start <- c(dob + adi, adi, 0)
     end   <- start + (maxDur-1) * c(1, 1, 1)
     lineIntgrl <- lineIntegral(start, end, m1Arr, timeRes = dt)
     F3 <- 1 - exp(-lineIntgrl$values)

     u <- runif(1)
     p <- which.min(u > F3)
     ade <- (lineIntgrl$points)[2, p-1] + (u - F3[p-1])*
                  ((lineIntgrl$points)[2, p] - (lineIntgrl$points)[2, p-1])/(F3[p] - F3[p-1])
     pmat.B[pid, 4] <- ade
   }
}


#total life time in base-case
sum(pmat.BC[ ,4])

#total life time in intervention A
sum(pmat.A[ ,4])

#total life time in intervention B
sum(pmat.B[ ,4])
